/* -------------------------------------------------------
                        ws2812_test.c

     Programm zum Ansprechen von WS2812 LED's

     MCU      :  STM8S103F3
     Hardware :  WS2812b Leuchtdiodenreihe
     Takt     :  interner Takt 16 MHz

     03.05.2017  R. Seelig
   ------------------------------------------------------ */

/*
##################################################################
                      STM8S103F3P6 Pinout
##################################################################


                            ------------
UART1_CK / TIM2_CH1 / PD4  |  1     20  |  PD3 / AIN4 / TIM2_CH2 / ADC_ETR
    UART1_TX / AIN5 / PD5  |  2     19  |  PD2 / AIN3
    UART1_RX / AIN6 / PD6  |  3     18  |  PD1 / SWIM
                     NRST  |  4     17  |  PC7 / SPI_MISO
              OSCIN / PA1  |  5     16  |  PC6 / SPI_MOSI
             OSCOUT / PA2  |  6     15  |  PC5 / SPI_CLK
                Vss (GND)  |  7     14  |  PC4 / TIM1_CH4 / CLK_CCO / AIN2
                VCAP (*1)  |  8     13  |  PC3 / TIM1_CH3 /
                Vdd (+Ub)  |  9     12  |  PB4 / I2C_SCL
           TIM2_CH3 / PA3  | 10     11  |  PB5 / I2C_SDA
                            -----------

*1 :  Ist mit min. 1uF gegen GND zu verschalten

*/

#include <stdint.h>
#include "stm8s.h"
#include "stm8_init.h"
#include "ws2812b.h"

#define black                   0
#define blue                    1
#define green                   2
#define cyan                    3
#define red                     4
#define magenta                 5
#define brown                   6
#define grey                    7
#define darkgrey                8
#define lightblue               9
#define lightgreen              10
#define lightcyan               11
#define lightred                12
#define lightmagenta            13
#define yellow                  14
#define white                   15


struct colvalue rgbcol;


/* -------------------------------------------------------------------------------------------
                                              M-A-I-N
   ------------------------------------------------------------------------------------------- */

int main(void)
{
  int8_t    i;

  sysclock_init(0);                     //  initialisieren mit internem RC-Taktgeber
  ws_init();
  ws_reset();


  ws_clrarray(&ledbuffer[0], ledanz);
  // LED-Nr. 3 insgesamt 5 mal blinken lassen mit der Farbe violett aus der
  // EGA-Farbpalette
  for (i= 0; i< 5; i++)
  {
    ws_setegacol(&ledbuffer[0], 3, lightmagenta);
    ws_showarray(&ledbuffer[0], ledanz);
    delay_ms(100);
    ws_clrarray(&ledbuffer[0], ledanz);
    ws_showarray(&ledbuffer[0], ledanz);
    delay_ms(100);
  }

  while(1)
  {

    ws_clrarray(&ledbuffer[0],ledanz);
    ws_showarray(&ledbuffer[0], ledanz);   // leeren LED-Strang anzeigen

    // fuegt eine blaue Leuchtdiode an Position 0 in den LED-Matrixpuffer
    // an Position 1 ein
    rgbfromvalue(0,0,30, &rgbcol);
    ws_setrgbcol(&ledbuffer[0], 0, &rgbcol);

    // und laesst die Matrix 2 Durchlaeufe nach rechts, dann nach links rotieren
    // (Lauflicht)
    for (i= 0; i< (ledanz * 2)-1; i++)
    {
      ws_showarray(&ledbuffer[0], ledanz);
      delay_ms(50);
      ws_buffer_rl(&ledbuffer[0], ledanz);
    }
    for (i= 0; i< (ledanz * 2)+1; i++)
    {
      ws_showarray(&ledbuffer[0], ledanz);
      delay_ms(50);
      ws_buffer_rr(&ledbuffer[0], ledanz);
    }
    delay_ms(1500);

    // laesst 4 mal hintereinander 4 verschiedene Farben aufblenden
    ws_clrarray(&ledbuffer[0],ledanz);
    for (i= 1; i< 3; i++)
    {
      rgbfromvalue(0,5,0, &rgbcol);
      ws_blendup_left(&ledbuffer[0], ledanz, &rgbcol, 40);
      rgbfromvalue(5,0,0, &rgbcol);
      ws_blendup_left(&ledbuffer[0], ledanz, &rgbcol, 40);
      rgbfromvalue(0,0,5, &rgbcol);
      ws_blendup_left(&ledbuffer[0], ledanz, &rgbcol, 40);
      rgbfromvalue(5,0,5, &rgbcol);
      ws_blendup_left(&ledbuffer[0], ledanz, &rgbcol, 40);
    }

    delay_ms(1000);

    // zeigt die ersten 7-EGA-Farben an
    for (i= 1; i< ledanz+1; i++)
    {
      ws_setegacol(&ledbuffer[0], i-1, i);
    }

    ws_reset();
    ws_showarray(&ledbuffer[0], ledanz);

    delay_ms(3000);
  }
}

